@@ -1,17 +1,16 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2022 -- leonerd@leonerd.org.uk
 
-package Parse::Man;
+package Parse::Man 0.03;
 
-use strict;
+use v5.14;
 use warnings;
+use utf8;
 
 use base qw( Parser::MGC );
 
-our $VERSION = '0.02';
-
 use constant pattern_ws => qr/[ \t]+/;
 
 =head1 NAME
@@ -29,6 +28,19 @@ building it as directed by these method invocations.
 
 =cut
 
+sub from_file
+{
+   my $self = shift;
+   my ( $file ) = @_;
+
+   if( $file =~ m/\.gz$/ ) {
+      return $self->SUPER::from_file( $file, binmode => ":gzip" );
+   }
+   else {
+      return $self->SUPER::from_file( $file );
+   }
+}
+
 sub parse
 {
    my $self = shift;
@@ -53,12 +65,40 @@ sub token_chunk
 {
    my $self = shift;
 
+   $self->skip_ws;
+
+   # A single chunk is either "quoted sequence" or whitespace-separated
    return $self->any_of(
-      sub { ( $self->expect( qr/"((?:\\.|[^"\\\n]+)*)(?:"|$)/m ) )[1] },
-      sub { $self->expect( qr/\S+/ ) },
+      sub { $self->committed_scope_of( '"', "_escaped", qr/"|$/m ) },
+      sub { $self->_escaped( 1 ) },
    );
 }
 
+sub _escaped
+{
+   my $self = shift;
+   my ( $break_on_space ) = @_;
+   my $ret = "";
+
+   my $consumed = 0;
+
+   $consumed++ while $self->any_of(
+      sub { my $esc = ( $self->expect( qr/\\(?|\((..)|(.))/ ) )[1];
+            $self->commit;
+            $ret .= $self->parse_escape( $esc );
+            1;
+          },
+      sub { length( my $more = $self->substring_before( $break_on_space ? qr/[\\\n\s]/ : qr/[\\\n]/ ) ) or return 0;
+            $ret .= $more;
+            1 },
+      sub { 0 },
+   );
+
+   $consumed or $self->fail( "Expected a chunk" );
+
+   return $ret;
+}
+
 sub parse_chunks
 {
    my $self = shift;
@@ -90,7 +130,10 @@ sub parse_line
       },
       sub {
          $self->commit;
-         $self->scope_of( undef, sub { $self->parse_plaintext }, "\n" );
+         $self->scope_of( undef, sub {
+            $self->{fonts} = [ "R" ];
+            $self->parse_plaintext;
+         }, "\n" );
       },
    );
 }
@@ -102,7 +145,9 @@ plain string value from the input content.
 
 =cut
 
-=head2 $parser->chunk( $text, %opts )
+=head2 chunk
+
+   $parser->chunk( $text, %opts )
 
 The C<%opts> hash contains the following options:
 
@@ -145,7 +190,12 @@ sub parse_directive_SM
 {
    my $self = shift;
    $self->_flush_para;
-   $self->chunk( $self->parse_chunks_flat, font => "R", size => -1 );
+
+   my @chunks = $self->parse_chunks;
+
+   $self->chunk( shift @chunks, font => "R", size =>  0 ) if @chunks > 2;
+   $self->chunk( shift @chunks, font => "R", size => -1 );
+   $self->chunk( shift @chunks, font => "R", size =>  0 ) if @chunks;
 }
 
 sub _parse_directive_alternate
@@ -207,7 +257,9 @@ chunks. Their return values are ignored.
 
 =cut
 
-=head2 $parser->para_TH( $name, $section )
+=head2 para_TH
+
+   $parser->para_TH( $name, $section )
 
 Handles the C<.TH> paragraph which gives the page title and section number.
 
@@ -220,7 +272,9 @@ sub parse_directive_TH
    $self->para_TH( $self->parse_chunks );
 }
 
-=head2 $parser->para_SH( $title )
+=head2 para_SH
+
+   $parser->para_SH( $title )
 
 Handles the C<.SH> paragraph, which gives a section header.
 
@@ -233,7 +287,9 @@ sub parse_directive_SH
    $self->para_SH( $self->parse_chunks_flat );
 }
 
-=head2 $parser->para_SS( $title )
+=head2 para_SS
+
+   $parser->para_SS( $title )
 
 Handles the C<.SS> paragraph, which gives a sub-section header.
 
@@ -246,7 +302,9 @@ sub parse_directive_SS
    $self->para_SS( $self->parse_chunks_flat );
 }
 
-=head2 $parser->para_TP( $opts )
+=head2 para_TP
+
+   $parser->para_TP( $opts )
 
 Handles a C<.TP> paragraph, which gives a term definition.
 
@@ -258,7 +316,9 @@ sub parse_directive_TP
    $self->_change_para( "TP" );
 }
 
-=head2 $parser->para_IP( $opts )
+=head2 para_IP
+
+   $parser->para_IP( $opts )
 
 Handles a C<.IP> paragraph, which is indented like the definition part of a
 C<.TP> paragraph.
@@ -269,9 +329,18 @@ sub parse_directive_IP
 {
    my $self = shift;
    $self->_change_para( "IP" );
+
+   if( defined( my $marker = $self->maybe( "token_chunk" ) ) ) {
+      $self->_change_para_options( marker => $marker );
+   }
+   if( defined( my $indent = $self->maybe( "token_chunk" ) ) ) {
+      $self->_change_para_options( indent => $indent );
+   }
 }
 
-=head2 $parser->para_P( $opts )
+=head2 para_P
+
+   $parser->para_P( $opts )
 
 Handles the C<.P>, C<.PP> or C<.LP> paragraphs, which are all synonyms for a
 plain paragraph content.
@@ -289,10 +358,36 @@ sub parse_directive_P
    *parse_directive_PP = *parse_directive_LP = \&parse_directive_P;
 }
 
+=head2 para_EX
+
+   $parser->para_EX( $opts )
+
+Handles the C<.EX> paragraph, which is example text; intended to be rendered
+in a fixed-width font without filling.
+
+=cut
+
+sub parse_directive_EX
+{
+   my $self = shift;
+   $self->_push_para( "EX" );
+}
+
+sub parse_directive_EE
+{
+   my $self = shift;
+   $self->_pop_para( "EX" );
+}
+
 sub parse_directive_RS
 {
    my $self = shift;
-   $self->_change_para_options( indent => "4n" );
+   if( defined( my $indent = $self->maybe( "token_chunk" ) ) ) {
+      $self->_change_para_options( indent => $indent );
+   }
+   else {
+      $self->_change_para_options( indent => "4n" );
+   }
 }
 
 sub parse_directive_RE
@@ -343,21 +438,93 @@ sub parse_plaintext
 {
    my $self = shift;
 
-   my @font = "R";
-
    $self->_flush_para;
 
    $self->sequence_of(
       sub { $self->any_of(
-         sub { $self->expect( qr/\\fP/ ); pop @font },
-         sub { push @font, ( $self->expect( qr/\\f([A-Z])/ ) )[1]; }, # \fX
-         sub { push @font, ( $self->expect( qr/\\f\((..)/ ) )[1]; },  # \f(AB
-         sub { my $else = ( $self->expect( qr/\\(.)/ ) )[1]; $self->chunk( $else, font => $font[-1], size => 0 ) },
-         sub { $self->chunk( $self->substring_before( qr/[\\\n]/ ), font => $font[-1], size => 0 ) },
+         sub { my $esc = ( $self->expect( qr/\\(?|\((..)|(.))/ ) )[1];
+               $self->commit;
+               my @chunks = $self->parse_escape( $esc );
+               $self->chunk( $_, font => $self->{fonts}[-1], size => 0 ) for @chunks;
+             },
+         sub { $self->chunk( $self->substring_before( qr/[\\\n]/ ), font => $self->{fonts}[-1], size => 0 ) },
       ); }
    );
 }
 
+sub parse_escape
+{
+   my $self = shift;
+   my ( $esc ) = @_;
+
+   my $meth = "parse_escape_$esc";
+   $meth = sprintf "parse_escape_x%v02X", $esc if length($esc) == 1 and $esc =~ m/[^A-Za-z0-9]/;
+   $meth = "parse_escape_char" if length($esc) > 1;
+   $meth = $self->can( $meth ) or
+   $self->fail( "Unrecognised escape sequence \\$esc" );
+   return $self->$meth( $esc );
+}
+
+sub parse_escape_x2D # \-
+{
+   my $self = shift;
+
+   # TODO: Unicode minus sign?
+   return "-";
+}
+
+# Ignore the "italic corrections" for now
+*parse_escape_x2C = # \,
+*parse_escape_x2F = # \/
+   sub { return };
+
+# The "empty" character
+sub parse_escape_x26 # \&
+{
+   my $self = shift;
+
+   return "";
+}
+
+*parse_escape_e = *parse_escape_E = *parse_escape_x5C = sub {
+   my $self = shift;
+
+   return "\\";
+};
+
+sub parse_escape_f
+{
+   my $self = shift;
+
+   $self->any_of(
+      sub { $self->expect( qr/P/ );
+            $self->commit;
+            @{ $self->{fonts} } > 1 or $self->fail( "Cannot \\fP without a \\f font defined" );
+            pop @{ $self->{fonts} }; },
+      sub { push @{ $self->{fonts} }, ( $self->expect( qr/([A-Z])/ ) )[1]; },
+      sub { push @{ $self->{fonts} }, ( $self->expect( qr/\((..)/ ) )[1]; },
+   );
+
+   return; # empty
+}
+
+# TODO: Vastly expand this table
+my %chars = (
+   'aq' => q('),
+   'bu' => '•',
+   'co' => '©',
+);
+
+sub parse_escape_char
+{
+   my $self = shift;
+   my ( $name ) = @_;
+
+   my $char = $chars{$name} // "<char $name>";
+
+   return $char;
+}
+
 sub _change_para
 {
    my $self = shift;
@@ -391,6 +558,30 @@ sub _flush_para
    }
 }
 
+sub _push_para
+{
+   my $self = shift;
+   my ( $new_mode ) = @_;
+
+   # Shallow clone
+   push @{ $self->{para_stack} }, { %{ $self->{para_options} } };
+   $self->_change_para( $new_mode );
+}
+
+sub _pop_para
+{
+   my $self = shift;
+   my ( $expect_mode ) = @_;
+
+   $self->{para_options}{mode} eq $expect_mode or
+      $self->fail( "Expected current paragraph mode of $expect_mode" );
+
+   $self->{para_options} = pop @{ $self->{para_stack} };
+
+   $self->_flush_para;
+   $self->{para_flushed} = 0;
+}
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
